import { defaults } from '../common/index.js';
import { checkNullArgument } from '../utils/index.js';
export class ContainerBuilder {
    constructor(_registrationBaseService, _instanceRegistrationService, _internalContainerService, _containerService) {
        this._registrationBaseService = _registrationBaseService;
        this._instanceRegistrationService = _instanceRegistrationService;
        this._internalContainerService = _internalContainerService;
        this._containerService = _containerService;
        this._registrations = [];
    }
    register(service) {
        checkNullArgument(service, 'service');
        const registrationBase = this._registrationBaseService.create(service);
        const registration = this._instanceRegistrationService.create(registrationBase);
        setDefaults(registrationBase);
        this._registrations.push(registrationBase);
        return registration;
    }
    build() {
        const registrations = this._registrations.slice(0);
        const internalContainer = this._internalContainerService.create();
        const container = this._containerService.create(internalContainer);
        internalContainer.add(registrations);
        return container;
    }
    copy(builder) {
        checkNullArgument(builder, 'builder');
        builder
            ._registrations
            .forEach((item) => {
            const clone = item.clone();
            const index = this._registrations
                .findIndex((registration) => registration.service === item.service);
            if (index < 0) {
                this._registrations.push(clone);
            }
            else {
                this._registrations[index] = clone;
            }
        });
    }
}
function setDefaults(registration) {
    registration.scope = defaults.scope;
    registration.owner = defaults.owner;
}
