import { checkDependency, checkNullArgument, isArray } from '../utils/index.js';
import { ApplicationError } from '../exceptions/index.js';
export class Api {
    constructor(_container) {
        this._container = _container;
        this._service = undefined;
        this._cache = {
            use: false,
            name: undefined
        };
        this._dependencies = [];
        this._attempt = false;
        this._args = [];
        this.args = this.args.bind(this);
        this.attempt = this.attempt.bind(this);
        this.name = this.name.bind(this);
        this.dependencies = this.dependencies.bind(this);
        this.cache = this.cache.bind(this);
        this.exec = this.exec.bind(this);
        this.execAsync = this.execAsync.bind(this);
    }
    get serviceValue() {
        if (this._service === null || this._service === undefined) {
            throw new ApplicationError({ message: 'Service was not initialized' });
        }
        return this._service;
    }
    get nameValue() {
        return this._name;
    }
    get cacheValue() {
        return this._cache;
    }
    get dependenciesValue() {
        return this._dependencies;
    }
    get isDependenciesResolvable() {
        return this._dependencies && this._dependencies.length > 0;
    }
    get attemptValue() {
        return this._attempt;
    }
    get throwResolveError() {
        return !this.attemptValue;
    }
    get argsValue() {
        return this._args;
    }
    service(value) {
        checkNullArgument(value, 'value');
        this._service = value;
        return {
            args: this.args,
            attempt: this.attempt,
            name: this.name,
            dependencies: this.dependencies,
            cache: this.cache,
            exec: this.exec,
            execAsync: this.execAsync
        };
    }
    args(...args) {
        this._args = args;
        return {
            attempt: this.attempt,
            name: this.name,
            dependencies: this.dependencies,
            cache: this.cache,
            exec: this.exec,
            execAsync: this.execAsync
        };
    }
    attempt() {
        this._attempt = true;
        return {
            name: this.name,
            dependencies: this.dependencies,
            cache: this.cache,
            exec: this.exec,
            execAsync: this.execAsync
        };
    }
    name(value) {
        checkNullArgument(value, 'value');
        this._name = value;
        return {
            dependencies: this.dependencies,
            cache: this.cache,
            exec: this.exec,
            execAsync: this.execAsync
        };
    }
    dependencies(data) {
        checkNullArgument(data, 'data');
        if (isArray(data)) {
            data
                .forEach(item => checkDependency(item));
            this._dependencies.push.apply(this._dependencies, data);
        }
        else {
            checkDependency(data);
            this._dependencies.push(data);
        }
        return {
            dependencies: this.dependencies,
            cache: this.cache,
            exec: this.exec,
            execAsync: this.execAsync
        };
    }
    cache(name) {
        this._cache.use = true;
        this._cache.name = name;
        return {
            exec: this.exec,
            execAsync: this.execAsync
        };
    }
    exec() {
        return this._container.execute(this);
    }
    async execAsync() {
        return new Promise(resolve => { resolve(this.exec()); });
    }
}
