import { checkNullArgument, checkDependency } from '../utils/index.js';
import { ResolutionError } from '../exceptions/index.js';
export const container = (_container) => {
    const result = {
        get cache() {
            return _container.cache;
        },
        createChild() {
            return container(_container.createChild());
        },
        dispose() {
            _container.dispose();
        },
        async disposeAsync() {
            return new Promise(resolve => {
                _container.dispose();
                resolve();
            });
        },
        resolve(service, ...args) {
            checkNullArgument(service, 'service');
            if (!args.length) {
                return _container.resolve(service);
            }
            return _container.resolve(service, ...args);
        },
        async resolveAsync(service, ...args) {
            return new Promise(resolve => {
                resolve(result.resolve(service, ...args));
            });
        },
        tryResolve(service, ...args) {
            checkNullArgument(service, 'service');
            if (!args.length) {
                return _container.tryResolve(service) || undefined;
            }
            return _container.tryResolve(service, ...args);
        },
        async tryResolveAsync(service, ...args) {
            return new Promise(resolve => {
                resolve(result.tryResolve(service, ...args) || undefined);
            });
        },
        resolveNamed(service, name, ...args) {
            checkNullArgument(service, 'service');
            checkNullArgument(name, 'name');
            if (!args.length) {
                return _container.resolveNamed(service, name);
            }
            return _container.resolveNamed(service, name, ...args);
        },
        async resolveNamedAsync(service, name, ...args) {
            return new Promise(resolve => {
                resolve(result.resolveNamed(service, name, ...args));
            });
        },
        tryResolveNamed(service, name, ...args) {
            checkNullArgument(service, 'service');
            checkNullArgument(name, 'name');
            const result = args.length <= 0 ?
                _container.tryResolveNamed(service, name) :
                _container.tryResolveNamed(service, name, ...args);
            return result || undefined;
        },
        async tryResolveNamedAsync(service, name, ...args) {
            return new Promise(resolve => {
                resolve(result.tryResolveNamed(service, name, ...args) || undefined);
            });
        },
        resolveWithDependencies(service, dependencies) {
            checkNullArgument(service, 'service');
            if (!dependencies || dependencies.length <= 0) {
                throw new ResolutionError({ message: 'No dependencies provided' });
            }
            dependencies.forEach(checkDependency);
            return _container.resolveWithDependencies(service, dependencies);
        },
        async resolveWithDependenciesAsync(service, dependencies) {
            return new Promise(resolve => {
                resolve(result.resolveWithDependencies(service, dependencies));
            });
        },
        resolveWith(service) {
            checkNullArgument(service, 'service');
            return _container.resolveWith(service);
        }
    };
    return result;
};
