import { checkNullArgument } from '../utils/index.js';
import { scope } from '../common/index.js';
export class RegistrationApi {
    constructor(_register) {
        this._register = _register;
        this.initializeBy = this.initializeBy.bind(this);
        this.lazy = this.lazy.bind(this);
        this.dispose = this.dispose.bind(this);
        this.named = this.named.bind(this);
        this.within = this.within.bind(this);
        this.transient = this.transient.bind(this);
        this.singleton = this.singleton.bind(this);
        this.instancePerContainer = this.instancePerContainer.bind(this);
        this.register = this.register.bind(this);
        this._isLazy = false;
    }
    get service() {
        return this._service;
    }
    get name() {
        return this._name;
    }
    get scope() {
        return this._scope;
    }
    get initializedBy() {
        return this._initializedBy;
    }
    get isLazy() {
        return this._isLazy;
    }
    get disposedBy() {
        return this._disposedBy;
    }
    provide(service) {
        checkNullArgument(service, 'service');
        const result = this.provideUndefined();
        this._service = service;
        return result;
    }
    provideUndefined() {
        this._service = undefined;
        return {
            initializeBy: this.initializeBy,
            lazy: this.lazy,
            dispose: this.dispose,
            named: this.named,
            within: this.within,
            transient: this.transient,
            singleton: this.singleton,
            instancePerContainer: this.instancePerContainer,
            register: this.register
        };
    }
    initializeBy(action) {
        checkNullArgument(action, 'action');
        this._initializedBy = action;
        return {
            lazy: this.lazy,
            dispose: this.dispose,
            named: this.named,
            within: this.within,
            transient: this.transient,
            singleton: this.singleton,
            instancePerContainer: this.instancePerContainer,
            register: this.register
        };
    }
    lazy() {
        this._isLazy = true;
        return {
            named: this.named,
            within: this.within,
            transient: this.transient,
            singleton: this.singleton,
            instancePerContainer: this.instancePerContainer,
            register: this.register
        };
    }
    dispose(action) {
        checkNullArgument(action, 'action');
        this._disposedBy = action;
        return {
            named: this.named,
            within: this.within,
            transient: this.transient,
            singleton: this.singleton,
            instancePerContainer: this.instancePerContainer,
            register: this.register
        };
    }
    named(name) {
        checkNullArgument(name, 'name');
        this._name = name;
        return {
            within: this.within,
            transient: this.transient,
            singleton: this.singleton,
            instancePerContainer: this.instancePerContainer,
            register: this.register
        };
    }
    within(scope) {
        checkNullArgument(scope, 'scope');
        this._scope = scope;
        return {
            register: this.register
        };
    }
    transient() {
        return this.within(scope.none);
    }
    singleton() {
        return this.within(scope.hierarchy);
    }
    instancePerContainer() {
        return this.within(scope.container);
    }
    register() {
        return this._register(this);
    }
}
