import { ApplicationError } from './application-error.js';
import { setPrototypeOf } from './common.js';
/**
 * Represents circular dependency error. It is thrown when two or more service
 * resolutions depend on each other directly on indirectly
 * @public
 */
export class CircularDependencyError extends ApplicationError {
    /**
     * Constructor
     * @param serviceName - service name
     * @param data - optional additional information about the error
     * @returns - an instance of CircularDependencyError
     */
    constructor(serviceName, data) {
        super({ message: `Circular dependency for service: ${serviceName}`, data });
        setPrototypeOf(this, CircularDependencyError.prototype);
        this.name = 'Circular Dependency Error';
        this._serviceName = serviceName;
    }
    /**
     * The name of the service that circularly resolves itself
     */
    get serviceName() {
        return this._serviceName;
    }
}
