import { ApplicationError } from './application-error.js';
import { setPrototypeOf } from './common.js';
/**
 * Represents resolution error. It is thrown when an error happens
 * during service resolution / instantiation
 * @public
 */
export class ResolutionError extends ApplicationError {
    /**
     * Constructor
     * @param params - optional additional information about the error
     * @returns - an instance of ResolutionError
     */
    constructor(params) {
        super(params);
        setPrototypeOf(this, ResolutionError.prototype);
        this.name = 'Resolution Error';
        this._innerError = params ? params.error : undefined;
    }
    /**
     * Internal error
     */
    get innerError() {
        return this._innerError;
    }
}
