import { callInfo } from '../common/index.js';
import { createProxy, getPropertyType, isBlackListProperty } from './common.js';
import { propertyType } from './types/index.js';
import { getAllPropertyNames, getPropertyDescriptor } from '../utils/index.js';
export class Proxy {
    constructor(_decorator) {
        this._decorator = _decorator;
        this.restrictedProperties = getAllPropertyNames(Function);
        this.propTypeToDescriptor = {
            [propertyType.method]: [callInfo.any, callInfo.method],
            [propertyType.getter]: [callInfo.any, callInfo.getter],
            [propertyType.setter]: [callInfo.any, callInfo.setter],
            [propertyType.fullProperty]: [
                callInfo.any,
                callInfo.getter,
                callInfo.setter,
                callInfo.getterSetter
            ],
            [propertyType.field]: [
                callInfo.any,
                callInfo.getter,
                callInfo.setter,
                callInfo.getterSetter,
                callInfo.field
            ]
        };
    }
    byPrototype(parent, storage) {
        const source = parent.prototype;
        const proxy = createProxy(this, parent, storage);
        getAllPropertyNames(source)
            .filter(name => !isBlackListProperty(name))
            .map(p => this.createStrategyInfo(source, proxy.prototype, p, '_parent'))
            .forEach(s => this.decorateProperty(s, storage));
        Object.getOwnPropertyNames(parent)
            .filter(name => this.restrictedProperties.indexOf(name) === -1)
            .map(p => this.createStrategyInfo(parent, proxy, p))
            .forEach(s => this.decorateProperty(s, storage));
        return proxy;
    }
    byInstance(parent, storage) {
        const result = Object.create({});
        getAllPropertyNames(parent)
            .filter(name => !isBlackListProperty(name))
            .map(p => this.createStrategyInfo(parent, result, p))
            .forEach(s => this.decorateProperty(s, storage));
        return result;
    }
    decorateProperty(strategyInfo, storage) {
        if (storage) {
            const types = this.propTypeToDescriptor[strategyInfo.type];
            const substitute = storage.getSubstitutes(strategyInfo.name, types);
            strategyInfo.substitute = substitute || undefined;
            this._decorator.wrap(strategyInfo);
            return;
        }
        this._decorator.wrap(strategyInfo);
    }
    createStrategyInfo(source, destination, name, contextName) {
        const descriptor = getPropertyDescriptor(source, name);
        const propertyType = getPropertyType(descriptor);
        return {
            type: propertyType,
            descriptor,
            substitute: undefined,
            name,
            source: source,
            destination: destination,
            contextName
        };
    }
}
