import { registrationType } from './types/index.js';
import { owner } from '../common/index.js';
import { ApplicationError } from '../exceptions/index.js';
import { uuid, factoryValueKey, isArray } from '../utils/index.js';
export class RegistrationBase {
    constructor(_service, id) {
        this._service = _service;
        this._owner = owner.externals;
        this._args = [];
        this._params = [];
        this._dependenciesValue = [];
        this._isLazy = false;
        this.args = [];
        this.params = [];
        this._id = id || uuid();
    }
    get name() {
        return this._name;
    }
    set name(value) {
        this._name = value;
    }
    get service() {
        return this._service;
    }
    get scope() {
        return this._scope;
    }
    set scope(value) {
        this._scope = value;
    }
    get owner() {
        return this._owner;
    }
    set owner(value) {
        this._owner = value;
    }
    get initializer() {
        return this._initializer;
    }
    set initializer(value) {
        this._initializer = value;
    }
    get disposer() {
        return this._disposer;
    }
    set disposer(value) {
        this._disposer = value;
    }
    get args() {
        return this._args;
    }
    set args(value) {
        this._args = value;
    }
    get params() {
        return this._params;
    }
    set params(value) {
        this._params = value;
    }
    get container() {
        return this._container;
    }
    set container(value) {
        this._container = value;
    }
    get instance() {
        return this._instance;
    }
    set instance(value) {
        this._instance = value;
    }
    get registrationType() {
        if (!this._registrationType) {
            throw new ApplicationError({
                message: 'Unknown registration type',
                data: this._registrationType
            });
        }
        return this._registrationType;
    }
    get dependenciesValue() {
        return this._dependenciesValue;
    }
    set dependenciesValue(value) {
        this._dependenciesValue = isArray(value) ? value : [];
    }
    get factory() {
        return this._factory;
    }
    set factory(value) {
        this._factory = value;
        this._registrationType = registrationType.factory;
    }
    get factoryType() {
        return this._factoryType;
    }
    set factoryType(value) {
        this._factoryType = value;
        this._registrationType = registrationType.factoryType;
    }
    get factoryValue() {
        return this._factoryValue;
    }
    set factoryValue(value) {
        this._factoryValue = value;
        this._registrationType = registrationType.factoryValue;
    }
    get id() {
        return this._id;
    }
    get isLazy() {
        return this._isLazy;
    }
    set isLazy(value) {
        this._isLazy = value;
    }
    cloneFor(container) {
        const result = this.clone();
        result.container = container;
        return result;
    }
    clone() {
        const result = new RegistrationBase(this._service, this._id);
        result._factory = this._factory;
        result._factoryType = this._factoryType;
        result._factoryValue = this._factoryValue;
        result._registrationType = this._registrationType;
        result.owner = this._owner;
        result.scope = this._scope;
        result.initializer = this._initializer;
        result.isLazy = this._isLazy;
        result.params = this._params;
        result.dependenciesValue = this._dependenciesValue;
        return result;
    }
    copyDependency(dependency) {
        this.name = dependency.named;
        this.initializer = dependency.initializer;
        if (dependency.factory) {
            this.factory = dependency.factory;
        }
        if (dependency.factoryType) {
            this.factoryType = dependency.factoryType;
        }
        if (factoryValueKey in dependency) {
            this.factoryValue = dependency.factoryValue;
        }
    }
    checkRegistrationType() {
        /// throws exception when no type
        this.registrationType;
    }
}
