import { scope, owner } from '../common/index.js';
import { checkNullArgument } from '../utils/index.js';
export class Registration {
    constructor(_base) {
        this._base = _base;
        this.initializeBy = this.initializeBy.bind(this);
        this.dispose = this.dispose.bind(this);
        this.named = this.named.bind(this);
        this.name = this.name.bind(this);
        this.within = this.within.bind(this);
        this.ownedBy = this.ownedBy.bind(this);
        this.transient = this.transient.bind(this);
        this.singleton = this.singleton.bind(this);
        this.instancePerContainer = this.instancePerContainer.bind(this);
        this.lazy = this.lazy.bind(this);
    }
    as(factory) {
        checkNullArgument(factory, 'factory');
        this._base.factory = factory;
        return {
            initializeBy: this.initializeBy,
            lazy: this.lazy,
            dispose: this.dispose,
            named: this.named,
            within: this.within,
            transient: this.transient,
            singleton: this.singleton,
            instancePerContainer: this.instancePerContainer
        };
    }
    asType(type, ...params) {
        checkNullArgument(type, 'type');
        this._base.factoryType = type;
        this._base.params = params;
        return {
            initializeBy: this.initializeBy,
            lazy: this.lazy,
            dispose: this.dispose,
            named: this.named,
            within: this.within,
            transient: this.transient,
            singleton: this.singleton,
            instancePerContainer: this.instancePerContainer
        };
    }
    asSelf(...params) {
        return this.asType(this._base.service, ...params);
    }
    asValue(value) {
        checkNullArgument(value, 'value');
        this._base.factoryValue = value;
        this._base.owner = owner.externals;
        this._base.scope = scope.none;
        return {
            named: this.name
        };
    }
    name(value) {
        this.named(value);
    }
    named(value) {
        checkNullArgument(value, 'value');
        this._base.name = value;
        return {
            within: this.within,
            transient: this.transient,
            singleton: this.singleton,
            instancePerContainer: this.instancePerContainer
        };
    }
    within(scope) {
        checkNullArgument(scope, 'scope');
        this._base.scope = scope;
    }
    transient() {
        return this.within(scope.none);
    }
    singleton() {
        return this.within(scope.hierarchy);
    }
    instancePerContainer() {
        return this.within(scope.container);
    }
    ownedBy(owner) {
        checkNullArgument(owner, 'owner');
        this._base.owner = owner;
    }
    initializeBy(action) {
        checkNullArgument(action, 'action');
        this._base.initializer = action;
        return {
            lazy: this.lazy,
            dispose: this.dispose,
            named: this.named,
            within: this.within,
            transient: this.transient,
            singleton: this.singleton,
            instancePerContainer: this.instancePerContainer
        };
    }
    dispose(action) {
        checkNullArgument(action, 'action');
        this._base.disposer = action;
        this.ownedBy(owner.container);
        return {
            named: this.named,
            within: this.within,
            transient: this.transient,
            singleton: this.singleton,
            instancePerContainer: this.instancePerContainer
        };
    }
    lazy() {
        this._base.isLazy = true;
        return {
            named: this.named,
            within: this.within,
            transient: this.transient,
            singleton: this.singleton,
            instancePerContainer: this.instancePerContainer
        };
    }
}
