import { InternalStorage, RegistrationStorage, DisposableStorage, ResolutionCache } from '../storage/index.js';
import { RegistrationBase, Registration } from '../registration/index.js';
import { container, Api as ContainerApi, ContainerBuilder, InternalContainer, Invoker } from '../build/index.js';
import { Decorator, RegistrationApi as DecoratorRegistrationApi, ResolutionApi as DecoratorResolutionApi } from '../decorators/index.js';
import { Decorator as InterceptorDecorator, Interceptor, Proxy } from '../interceptors/index.js';
export class Scaffold {
    createBuilder() {
        const internalContainerService = this.internalContainerService();
        return this.builderService().create(internalContainerService);
    }
    createDecorator() {
        const decoratorRegistrationService = this.decoratorRegistrationApiService();
        const internalStorageService = this.internalStorageService();
        const internalContainerService = this.internalContainerService();
        return new Decorator(this.builderService(), internalContainerService, decoratorRegistrationService, internalStorageService);
    }
    createInterceptor() {
        const decorator = new InterceptorDecorator();
        const proxy = new Proxy(decorator);
        return new Interceptor(proxy);
    }
    builderService() {
        return {
            create: (internalContainerService) => {
                const baseRegistrationService = this.registrationBaseService();
                const instanceRegistrationService = this.instanceRegistrationService();
                const containerService = this.containerService();
                return new ContainerBuilder(baseRegistrationService, instanceRegistrationService, internalContainerService, containerService);
            }
        };
    }
    internalStorageService() {
        return {
            create() {
                return new InternalStorage();
            }
        };
    }
    registrationStorageService() {
        const service = {
            create() {
                return new InternalStorage();
            }
        };
        return {
            create() {
                return new RegistrationStorage(service);
            }
        };
    }
    disposableStorageService() {
        return {
            create() {
                return new DisposableStorage();
            }
        };
    }
    resolutionCacheService() {
        return {
            create() {
                return new ResolutionCache();
            }
        };
    }
    registrationBaseService() {
        return {
            create(service) {
                return new RegistrationBase(service);
            }
        };
    }
    containerService() {
        return {
            create: container
        };
    }
    containerApiService() {
        return {
            create: (container) => {
                return new ContainerApi(container);
            }
        };
    }
    instanceRegistrationService() {
        return {
            create: (baseRegistration) => {
                return new Registration(baseRegistration);
            }
        };
    }
    internalContainerService() {
        const dependencies = () => {
            const registrationStorageService = this.registrationStorageService();
            const disposableStorageService = this.disposableStorageService();
            const baseRegistrationService = this.registrationBaseService();
            const containerApiService = this.containerApiService();
            const invokerService = this.invokerService();
            const resolutionCacheService = this.resolutionCacheService();
            return {
                registrationStorageService,
                disposableStorageService,
                baseRegistrationService,
                containerApiService,
                invokerService,
                resolutionCacheService
            };
        };
        return {
            create() {
                const params = dependencies();
                return new InternalContainer(params.registrationStorageService, params.disposableStorageService, params.baseRegistrationService, params.containerApiService, params.invokerService, params.resolutionCacheService, this.resolutionDetails);
            }
        };
    }
    invokerService() {
        return {
            create(container, resolutionDetails) {
                return new Invoker(container, resolutionDetails);
            }
        };
    }
    decoratorRegistrationApiService() {
        return {
            createRegistration(register) {
                return new DecoratorRegistrationApi(register);
            },
            createResolution(resolve) {
                return new DecoratorResolutionApi(resolve);
            }
        };
    }
}
