import { registrationType } from '../registration/index.js';
import { empty } from './store.js';
export class RegistrationStorage {
    constructor(storageService) {
        this._addStrategy = [];
        this._getStrategy = [];
        this._internalStorage = storageService.create();
        this._addStrategy[registrationType.factoryType] = this.addForFactoryType.bind(this);
        this._addStrategy[registrationType.factory] = this.addForFactory.bind(this);
        this._addStrategy[registrationType.factoryValue] = this.addForFactoryValue.bind(this);
        this._getStrategy[registrationType.factoryType] =
            this.getForFactoryType.bind(this);
        this._getStrategy[registrationType.factory] =
            this.getForFactory.bind(this);
        this._getStrategy[registrationType.factoryValue] =
            this.getForFactoryValue.bind(this);
    }
    addEntry(registration) {
        const strategy = this._addStrategy[registration.registrationType];
        strategy(registration);
    }
    getEntry(registration) {
        const storage = this._internalStorage.tryGet(registration.service);
        if (!storage)
            return undefined;
        return this._getStrategy[storage.type](registration, storage);
    }
    clear() {
        this._internalStorage.clear();
    }
    addForFactoryType(registration) {
        const storage = this._internalStorage.register(registration.service, empty.typeFactoryBucket);
        if (!registration.name) {
            storage.factory.noName = registration;
        }
        else {
            storage.factory.names[registration.name] = registration;
        }
    }
    addForFactory(registration) {
        const storage = this._internalStorage.register(registration.service, empty.factoryBucket);
        const argsCount = this.getArgumentsCount(registration);
        if (!registration.name) {
            storage.factory.noName[argsCount] = registration;
        }
        else {
            const bucket = storage.factory.names[registration.name] || {};
            bucket[argsCount] = registration;
            storage.factory.names[registration.name] = bucket;
        }
    }
    addForFactoryValue(registration) {
        const storage = this._internalStorage.register(registration.service, empty.valueFactoryBucket);
        storage.type = registrationType.factoryValue;
        if (!registration.name) {
            storage.factory.noName = registration;
        }
        else {
            storage.factory.names[registration.name] = registration;
        }
    }
    getForFactoryType(registration, storage) {
        return !registration.name ? storage.factory.noName :
            storage.factory.names[registration.name];
    }
    getForFactory(registration, storage) {
        const argsCount = this.getArgumentsCount(registration);
        const name = registration.name ? storage.factory.names[registration.name] : undefined;
        return registration.name ?
            (name ? name[argsCount] : undefined) :
            storage.factory.noName[argsCount];
    }
    getForFactoryValue(registration, storage) {
        return !registration.name ? storage.factory.noName :
            storage.factory.names[registration.name];
    }
    getArgumentsCount(registration) {
        return registration.factory ?
            this.getFactoryArgsCount(registration.factory) :
            registration.args.length;
    }
    getFactoryArgsCount(factory) {
        const paramsCount = factory.length;
        return Math.max(paramsCount - 1, 0);
    }
}
