import { InternalStorage } from './internal-storage.js';
import { ResolutionError } from '../exceptions/index.js';
export class ResolutionCache {
    constructor() {
        this._instance = {};
        this._storage = new InternalStorage();
    }
    get instance() {
        return this._instance;
    }
    resolve(name) {
        if (!this._storage.contains(name)) {
            throw new ResolutionError({ message: 'Could not resolve service', data: name });
        }
        return this._storage.get(name);
    }
    add(name, value) {
        this._storage.add(name, value);
        Object.defineProperty(this.instance, name, {
            enumerable: true,
            configurable: false,
            get: () => {
                return this._storage.get(name);
            }
        });
    }
    clear() {
        this._storage.clear();
        this._instance = {};
    }
}
