import { checkNullArgument } from './common.js';
import { isArray } from './reflection.js';
import { ArgumentError } from '../exceptions/index.js';
export class ImmutableArray {
    constructor(data) {
        checkNullArgument(data, 'data');
        if (!isArray(data)) {
            throw new ArgumentError('data', { message: 'should represent an array' });
        }
        this._data = this.initialize(data);
    }
    get value() {
        return this._data.slice(0);
    }
    static createImmutable(array) {
        return new ImmutableArray(array);
    }
    initialize(data) {
        return data.map(item => {
            const type = typeof (item);
            return type === 'object' ||
                type === 'function' ? Object.freeze(item) : item;
        });
    }
}
